/* Vertex shader */
 
uniform float height;
uniform int stage;
uniform float level;
uniform vec4 RGB;
uniform int MODE;
uniform vec4 METRICS;

float RGB2HUE(in float RR, in float GG, in float BB){
	float MMAX = max((RR, GG), BB);
	float MMIN = min(min(RR, GG), BB);
	float l = (MMAX + MMIN) / 2.0;
	float h = l;
	float s = l;
	float d = MMAX - MMIN;
	
	if (MMAX == MMIN) {
		h = s = 0.0; // achromatic
	} else {
	
	
		if(l > 0.5){
			s = d / (2.0 - MMAX - MMIN);
		}else{
			s = d / (MMIN + MMIN);
		}
	
		//s = l > 0.5 ? d / (2.0 - MMAX - MMIN) : d / (MMIN + MMIN);
	
		if(MMAX == RR){
			float f;
			if(GG<BB){
				f = 6.0;
			}else{
				f = 0.0;
			}
    			h = (GG - BB) / d + f;
    		}
    		
    		if(MMAX == GG){
    			h = (BB - RR) / d + 2.0;
    		}
    		
    		
    		if(MMAX == BB){
    			h = (RR - GG) / d + 4.0;
    		}
        	h = h / 6.0;
    	}

	return h;
};



float HUE2RGB(in float p, in float q, in float t){
	
	if(t < 0.0) t += 1.0;
	if(t > 1.0) t -= 1.0;
	if(t < 1.0/6.0) return p + (q - p) * 6.0 * t;
	if(t < 1.0/2.0) return q;
	if(t < 2.0/3.0) return p + (q - p) * (2.0/3.0 - t) * 6.0;
	return p;
}

void HLS2RGB(in float hue, in float saturation, in float lightness, out float red, out float green, out float blue){

	float q;
	float p;

    if(saturation == 0.0){	// achromatic
        red = lightness;
        green = lightness;
        blue = lightness;
    }else{


        q = lightness < 0.5 ? lightness * (1.0 + saturation) : lightness + saturation - lightness * saturation;
        p = 2.0 * lightness - q;
        red = HUE2RGB(p, q, hue + 1.0/3.0);
        green = HUE2RGB(p, q, hue);
        blue = HUE2RGB(p, q, hue - 1.0/3.0);
    }
}

void main(void)
{
	vec4 v = vec4(gl_Vertex); 
	
	//v[0] *= 2.0;
	//v[1] *= 2.0;
	vec4 c,b;
        
	float h = (2.0/3.0) - (1.0-(v.y/height))*(2.0/3.0);
	float red;
	float green;
	float blue;
	
	HLS2RGB(h,1.0,0.5,red,green,blue);
        
	c.r = red;
	c.g = green;
	c.b = blue;
	c.a = 1.0;

	b.a = 0.7;
	b.r = 0.0;
	b.g = 0.0;
	b.b = 0.0;
	
	 
	if(stage == 0){
		
		gl_FrontColor = c;
		gl_BackColor = c;

	}else{
		
		c.r = 1.0;
		c.g = 1.0;
		c.b = 1.0;
		c.a = 1.0;
	
	}
	float A;

	float x1;
	float x2;
	float d;

	switch(MODE){
	case 1:
		// REGULAR DRAWING OF TARGET RESPONSES

		// SORT-OUT COLOUR AND ALPHA
		// THIS GENERATE A FADE TOWARDS THE EDGE OF THE CIRCLE
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		A = RGB[3] * 1.0;
//		c[3] = A;
		c[3] = (2.0*A) - (2.0*A * sqrt((v[0]*v[0])+(v[1]*v[1])) / 100.0);


//float H = RGB2HUE(RGB[0],RGB[1],RGB[2]);
//float dB = (20.0 * (log(METRICS[3])/log(10))) / -60.f;
//HLS2RGB(H,0.5+(METRICS[2]*0.005*METRICS[3]),0.5,red,green,blue);
//c[0] = red;
//c[1] = green;
//c[2] = blue;



		// SCALE THE VERTEX
		v[0] *= METRICS[2]*0.01*METRICS[3] * 1.0;
		v[1] *= METRICS[2]*0.01*METRICS[3] * 1.0;

		//v[0] *= METRICS[2]*0.1*METRICS[3] * 1.0;
		//v[1] *= METRICS[2]*0.1*METRICS[3] * 1.0;

		// POSITION THE VERTEX
		v[0] += METRICS[0];
		v[1] += METRICS[1];

		// GIVE IT A COLOUR
		gl_FrontColor = c;
		gl_BackColor = c;

		 // METRICS[2]);//METRICS[2]*0.005*METRICS[3];
		 break;
	case 3:
		// DRAW THE OUTLINE FOR RESPONSE ...
		
		A = RGB[3] * 1.0;
		x1 = v[0];// - METRICS[3];
		x2 = v[1];// - METRICS[3];

		d = sqrt((x1*x1)+(x2*x2));
		d = abs(100-d);
//		d = d*1;

		c[3] = 1.0-d;
		
		v[0] *= METRICS[3] * 0.01;
		v[1] *= METRICS[3] * 0.01;
		v[0] += METRICS[0];
		v[1] += METRICS[1];

		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];

		gl_FrontColor = c;
		gl_BackColor = c;
	break;
		case 2:
		// DRAW THE OUTLINE FOR RESPONSE ...
		
		A = RGB[3] * 1.0;
		x1 = v[0];// - METRICS[3];
		x2 = v[1];// - METRICS[3];

		d = sqrt((x1*x1)+(x2*x2));
		d = abs(100-d);
//		d = d*1;

		c[3] = 1.0-d;
		
		v[0] *= METRICS[3] * 0.01;
		v[1] *= METRICS[3] * 0.01;
		v[0] += METRICS[0];
		v[1] += METRICS[1];

		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		c[3] = RGB[3];

		gl_FrontColor = c;
		gl_BackColor = c;
	break;
	case 5:
		// draw a picture

	break;
	case 6:
	
		// DRAW MOVING ENVELOPE RESPONSE
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		c[3] = RGB[3];
		c.a = 0.3 + (METRICS[3] * 0.7);
		if(METRICS[3] > 0.9999){
			c[0] = 1.0;
			c[1] = 0.0;
			c[2] = 0.0;
			c[3] = 1.0;
		}
		gl_FrontColor = c;
		gl_BackColor = c;
	break;
	case 7:
	// BACK OF ENVELOPE -> FLASHES RED IF OVERLOAD
		c[0] = RGB[0];
		c[1] = RGB[1];
		c[2] = RGB[2];
		c[3] = RGB[3];
		//c.a = 0.3 + (METRICS[3] * 0.7);
		if(METRICS[3] > 0.9999){
			c[0] = 1.0;
			c[1] = 0.0;
			c[2] = 0.0;
			c[3] = 1.0;
		}
		gl_FrontColor = c;
		gl_BackColor = c;
		break;
	case 0:
		gl_FrontColor = RGB;
		gl_BackColor = RGB;
		break;

	case 10:
	
		// HEAT MAP
		
		float D = RGB[3];
		float ALPHA = 0.7;
		if(D < 0.001){
			ALPHA = 0.0;
		}
		//D = D*D*D;
		D = sqrt(D);
		
		
		//D*=2.0;
		D*=3.0;
		D*=0.8;
		D+=0.67;
		D-=1.0;
		D*=-1.0;
		D+=0.2;
		D = max(-1.0,D);
		D = min(1.0,D);
		
		//float n = D * 2.0;
		//n=mod(n,1.0);
		
		//D = n+0.1;
		
		//D = sqrt(D);
		//D = sqrt(D);
		
		//D+=0.6;
		float L =1.0;
		L = min(L,1.0);
		L = max(L,0.0);
		D -= RGB[2];
		if(D<-1.0){
			D+=1.0;
		}
		HLS2RGB(D,L,0.5,red,green,blue);
		//HLS2RGB(0.5,0.5,0.5,red,green,blue);
		c[0] = red;
		c[1] = green;
		c[2] = blue;
		c[3] = ALPHA;
		
		gl_FrontColor = c;
		gl_BackColor = c;
	
	break;
	case 12:
		// FEEDBACK CABLES
		// VALUE IN ALPHA CHANNEL DETERMINES HUE -> ALPHA is IN dB
		// ANTHING OVER -0dB IS RED, ELSE 0->0.5 GREEN, THROUGH ORANGE
		float HH = RGB[3];
		float AA = 1.0;

		//if(HH < -60.0){
		//}else{
		
		HH = min(HH,0.0);
		HH = max(-60.0,HH);
		HH /= -120.0;

		HH = max(HH,0.0);

		// ALPHA VALUE FALL-OFF BELOW -30 FROM 1 DOWN TO 0.5;
		
		if(HH < 0.5){
			AA = 1.0 - (1.8 * HH);
		}
		if(HH > 0.499){
			AA = 0.2;
		}

		//};
		HLS2RGB(HH,1.0,0.5,red,green,blue);
		//HLS2RGB(0.5,0.5,0.5,red,green,blue);
		c[0] = red;
		c[1] = green;
		c[2] = blue;
		c[3] = AA;
		
		gl_FrontColor = c;
		gl_BackColor = c;

	break;
	
	}

	gl_Position = gl_ModelViewProjectionMatrix * v;   
	    
}

